#include <Arduino.h>
#include <Keypad.h>
#include <LedControl.h>
#include <string.h>
#include <stdio.h>
#include <LiquidCrystal_I2C.h>
#include <Wire.h>
#define CS 3
#define CLK 4
#define DIN 2
#define MAX_STEPS 7
#define SPEAKER 13
const byte ROWS = 4;
const byte COLS = 4;
char hexaKeys[ROWS][COLS] = {
    {'C', '8', '4', '0'},
    {'D', '9', '5', '1'},
    {'E', 'A', '6', '2'},
    {'F', 'B', '7', '3'}};

//c1 la 12, c2 - 11, c3 - 10, c4 - 9, r1 - 6
byte rowPins[ROWS] = {5, 6, 7, 8};
byte colPins[COLS] = {12, 11, 10, 9};
byte love[8]={ B00000000,B01100110,B10011001,B10011001,B10000001,B01000010,B00100100,B00011000};
byte three[8] = {B00000000,B01111110,B00000010,B00000010,B01111110,B00000010,B00000010,B01111110};
byte two[8] = {B00000000,B00011000,B00100100,B01000010,B00001100,B00111000,B01100000,B01111110};
byte one[8] = {B00000000,B00001000,B00011000,B00101000,B00001000,B00001000,B00001000,B00001000};
Keypad customKeypad = Keypad(makeKeymap(hexaKeys), rowPins, colPins, ROWS, COLS);
int zero = 0;
LedControl lc = LedControl(DIN, CLK, CS, 0);
LiquidCrystal_I2C lcd(0x27, 16, 2);
bool isSelectGame;
bool isUserTurn;
int steps;
int combinations[MAX_STEPS];
int buttonPresses;
bool isValidCombination;
bool isLevelCorrect;
int winGameSound[] = {262, 262, 392, 330, 262, 245, 220, 349, 349, 330, 262, 294, 262};
int loseGameSound[] = {262, 245, 233, 220, 208, 196, 185, 175, 165, 156, 147, 139, 131};
void setup()
{
  
  lcd.init();
  Serial.begin(9600);
  lc.shutdown(0, false);
  lc.setIntensity(0, 15);
  lc.clearDisplay(0);
  randomSeed(analogRead(0));
  isSelectGame = true;
  buttonPresses = 0;
  lcd.backlight();
  isLevelCorrect = true;
}


void loop()
{
  char levelGame = customKeypad.getKey();
  if(levelGame) {
    Serial.println(levelGame);
  }
  if (isSelectGame)
  {
    if(isLevelCorrect) {
      lcd.setCursor(0, 0);
      lcd.print("Select the level");
      lcd.setCursor(0, 1);
      lcd.print("using last row");
    }
    // momentul cand se selecteaza ce intensitate se doreste
    // si anume 4 niveluri, care vor fi selectate din butoanele cele
    // mai din jos ale matricii de butoane
    char levelGame = customKeypad.getKey();
    if(levelGame) {
      lcd.clear();
      if (levelGame == 'C' || levelGame == 'D' || levelGame == 'E' || levelGame == 'F') {
        isLevelCorrect = true;
      // se intra in modul de joc
        isSelectGame = false;
        if (levelGame == 'C')
        {
          steps = 3;
        }
        else if (levelGame == 'D')
        {
          steps = 4;
        }
        else if (levelGame == 'E')
        {
          steps = 5;
        }
        else if (levelGame == 'F')
        {
          steps = 6;
        }
        for (int i = 0; i < steps; i++)
        {
          combinations[i] = random(16);
        }
        isUserTurn = false;
        isValidCombination = true;
    } else {
        isLevelCorrect = false;
        lcd.setCursor(0, 0);
        lcd.print("Use just the");
        lcd.setCursor(0, 1);
        lcd.print("last row");
        // de afisat ca nu s-a selectat un nivel corespunzator
    }
    }
  }
  else
  {
    if (isUserTurn == false)
    {
      for (int i = 0; i < 8; i++) {
          lc.setRow(0, i, three[i]);
      }
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("The game will");
      lcd.setCursor(0, 1);
      lcd.print("start in 3");
      delay(1000);
      for (int i = 0; i < 8; i++) {
        lc.setRow(0, i, B00000000);
      }
      for (int i = 0; i < 8; i++) {
          lc.setRow(0, i, two[i]);
      }
      lcd.setCursor(9, 1);
      lcd.print("2");
      delay(1000);
      for (int i = 0; i < 8; i++) {
        lc.setRow(0, i, B00000000);
      }
      for (int i = 0; i < 8; i++) {
          lc.setRow(0, i, one[i]);
      }
      lcd.setCursor(9, 1);
      lcd.print("1");
      delay(1000);
      for (int i = 0; i < 8; i++) {
        lc.setRow(0, i, B00000000);
      }
      lcd.setCursor(9, 1);
      lcd.print("0");
      for (int i = 0; i < steps; i++)
      {
        lc.setLed(0, combinations[i] / ROWS * 2, combinations[i] % COLS * 2, true);
        lc.setLed(0, combinations[i] / ROWS * 2 + 1, combinations[i] % COLS * 2, true);
        lc.setLed(0, combinations[i] / ROWS * 2, combinations[i] % COLS * 2 + 1, true);
        lc.setLed(0, combinations[i] / ROWS * 2 + 1, combinations[i] % COLS * 2 + 1, true);
        delay(1000);
        lcd.clear();
        lc.setLed(0, combinations[i] / ROWS * 2, combinations[i] % COLS * 2, false);
        lc.setLed(0, combinations[i] / ROWS * 2 + 1, combinations[i] % COLS * 2, false);
        lc.setLed(0, combinations[i] / ROWS * 2, combinations[i] % COLS * 2 + 1, false);
        lc.setLed(0, combinations[i] / ROWS * 2 + 1, combinations[i] % COLS * 2 + 1, false);
        if(i != steps - 1) {
          delay(1000);
        }
      }
      isUserTurn = true;
      buttonPresses = 0;
    }
    else
    {
      // if care functioneaza ca un while
      if (buttonPresses < steps && isValidCombination) {
        lcd.setCursor(0, 0);
        lcd.print("Use the buttons");
        lcd.setCursor(0, 1);
        lcd.print("to reproduce it");
        char keyPressed = customKeypad.getKey();
        if (keyPressed) {
          int integerKeyPressed;
          if (strchr("0123456789", keyPressed)) {
            integerKeyPressed = keyPressed - '0';
          }
          else {
            integerKeyPressed = keyPressed - 'A' + 10;
          }
          if (combinations[buttonPresses] == integerKeyPressed) {
            buttonPresses++;
          } else {
            //a apasat un buton gresit
            isValidCombination = false;
            for (int i = 0; i < 8; i++) {
              lc.setRow(0, i, B11111111);
            }
            lcd.clear();
            lcd.setCursor(0, 0);
            lcd.print("You have failed");
            lcd.setCursor(0, 1);
            lcd.print("Please try again");
            for(int i = 0; i < 13; i++){
              tone(SPEAKER, loseGameSound[i]);
              delay(150);
            }
            noTone(SPEAKER);
            pinMode(SPEAKER, LOW);
            delay(3000);
            for (int i = 0; i < 8; i++) {
              lc.setRow(0, i, B00000000);
            }
            isSelectGame = true;
            lcd.clear();
          }
        }
      } else if (buttonPresses == steps && isValidCombination == true) {
        for (int i = 0; i < 8; i++) {
          lc.setRow(0, i, love[i]);
        }
            lcd.clear();
            lcd.setCursor(0, 0);
            lcd.print("Well done!");
            lcd.setCursor(0, 1);
            lcd.print("You are so smart");
            for (int i = 0; i < 13; i++) {
              tone(SPEAKER, winGameSound[i]);
              if (i == 1 || i == 8){
                delay(150);
              } else if (i == 6 || i == 12){
                delay(1000);
              } else {
                delay(400);
              }
              noTone(SPEAKER);
              delay(50);
            }
            noTone(SPEAKER);
            pinMode(SPEAKER, LOW);
            delay(3000);
            for (int i = 0; i < 8; i++) {
              lc.setRow(0, i, B00000000);
            }
        isSelectGame = true;
        lcd.clear();
      }
    }
  }
}